#include "specialprot.h"


//////// Special handlers ////////

void handleSpecialHeader(hostMessage * msg)
{
	if (msg->len >= 2)
	{
		msg->src->gotSpecialMessage(msg);
	}
}


//////// Core protocol handshake ////////

void handleEncryptRequest(hostMessage * msg)
{
	if (msg->len == 8)
	{
		msg->src->gotEncryptRequest(getLong(msg->msg, 2), getShort(msg->msg, 6));
	}
}

void handleEncryptResponse(hostMessage * msg)
{
	if (msg->len == 6)
	{
		msg->src->gotEncryptResponse(getLong(msg->msg, 2));
	}
	else if (msg->len == 7)
	{
		msg->src->gotEncryptResponse(getLong(msg->msg, 2), getByte(msg->msg, 6));
	}
}

void handleDisconnect(hostMessage * msg)
{
	if (msg->len == 2)
	{
		msg->src->gotDisconnect();
	}
}


//////// Reliable messaging ////////

void handleReliable(hostMessage * msg)
{
	if (msg->len >= 7)
	{
		msg->src->gotReliable(getLong(msg->msg, 2), msg->msg + 6, msg->len - 6);
	}
}

void handleACK(hostMessage * msg)
{
	if (msg->len == 6)
	{
		msg->src->gotACK(getLong(msg->msg, 2));
	}
}


//////// Time synchronization ////////

void handleSyncRequest(hostMessage * msg)
{
	if (msg->len == 2)
	{
		msg->src->gotSyncRequest();
	}
	else if (msg->len == 6)
	{
		msg->src->gotSyncRequest(getLong(msg->msg, 2));
	}
	else if (msg->len == 14)
	{
		msg->src->gotSyncRequest(getLong(msg->msg, 2), getLong(msg->msg, 6), getLong(msg->msg, 10));
	}
}

void handleSyncResponse(hostMessage * msg)
{
	if (msg->len == 10)
	{
		msg->src->gotSyncResponse(getLong(msg->msg, 2), getLong(msg->msg, 6));
	}
}


//////// Transmission modes ////////

void handleChunkBody(hostMessage * msg)
{
	if (msg->len >= 3)
	{
		msg->src->gotChunkBody(msg->msg + 2, msg->len - 2);
	}
}

void handleChunkTail(hostMessage * msg)
{
	if (msg->len >= 3)
	{
		msg->src->gotChunkTail(msg->msg + 2, msg->len - 2);
	}
}

void handleBigChunk(hostMessage * msg)
{
	if (msg->len >= 7)
	{
		msg->src->gotBigChunk(getLong(msg->msg, 2), msg->msg + 6, msg->len - 6);
	}
}

void handleCluster(hostMessage * msg)
{
	if (msg->len >= 6)
	{	// Require at least two messages to be clustered
		msg->src->gotCluster(msg->msg + 2, msg->len - 2);
	}
}
